#include <iostream>
#include <cstdarg>
#include <vector>
#include <cstdio>

using namespace std;

#include "fraction"
#include "verylong"

static int verbose;
static double maxerror = 1e-10; /* epsilon */

template<typename sourcetype>
class algofinder;

template<typename elem>
static ostream &operator << (ostream &dest, const vector<elem> &guess)
{
    typedef typename elem::srctype sourcetype;
    for(unsigned b=0; b<guess.size(); ++b)
    {
        if(guess[b].integer)
        {
            bool nega = guess[b].integer.negative();
            
            if(b)dest << ' ';
            
            sourcetype nom = guess[b].integer.nom();
            sourcetype denom=guess[b].integer.denom();
            
            if(nega ^ (nom < sourcetype(0))) { nom=-nom; denom=-denom; }
            
            if(nom == sourcetype(-1) && guess[b].exponent)
            {
                dest << '-';
                if(denom != sourcetype(1))dest << ' ';
            }
            else
            {
                if(b && nom >= sourcetype(0))dest << '+';
                if(abs(nom) > sourcetype(1) || !guess[b].exponent)dest << nom;
            }
            if(guess[b].exponent)
            {
                dest << 'x';
                if(guess[b].exponent == 2)
                    dest << '';
                else if(guess[b].exponent > 1)
                    dest << '^' << guess[b].exponent;
            }
            if(denom != sourcetype(1))
                dest << '/' << denom;
        }
    }
    return dest;
}

template<typename sourcetype>
class algofinder
{
private:
    typedef sourcetype tmptype;

    struct elem
    {
        typedef sourcetype srctype;
        tmptype integer;        /* Number this is multiplied by      */
        unsigned exponent;  /* 0=constant, 1=x, 2=xx, 3=xxx, ... */
    public:
        elem() : integer(0), exponent(0) { }
        elem(int add, unsigned exp) : integer(add), exponent(exp) { }
    };
    
    /* Return value: biggest error */
    tmptype Test(
        const vector<sourcetype> &keys,
        const vector<sourcetype> &values,
        const vector<algofinder<sourcetype>::elem> &guess) const
    {
        tmptype biggesterror = 0, error, result, thisval;
        
        /* Test the guess */

        for(unsigned a=0; a<keys.size(); ++a)
        {
            const tmptype &val = keys[a];
            result = (tmptype)0;
            
            for(unsigned b=0; b<guess.size(); ++b)
            {
                thisval = (tmptype)1;
                for(unsigned c=0; c<guess[b].exponent; ++c)thisval *= val;
                thisval *= guess[b].integer;
                result += thisval;
            }
            
            error = abs(result - (tmptype)values[a]);
            
            if(error > biggesterror) biggesterror = error;

/*
            if(error.value() > maxerror)
                break;            
*/
        }

        return biggesterror;
    }

    /* Non recursive */
    tmptype calcdet(const vector<sourcetype> &mat, unsigned depf) const
    {
        /* mat.size() should be depf * depf */
        
        vector<unsigned> ps(depf); /* position stack   */
        vector<unsigned> pc(depf); /* position counter */
        
        tmptype result = 0, thisresult;

        unsigned level=0, maxlevel=depf-1;
        ps[0] = pc[0] = 0;
        
        for(;;)
        {
            for(unsigned a=0; a<level; ++a)
                if(ps[a] == ps[level])
                    goto Rotate;

            if(level < maxlevel)
            {
                ++level;
                ps[level] = 0;
                pc[level] = 0;
                continue;
            }

           {
            bool nega = false;
            for(unsigned a=0; a<depf; ++a)
                if(pc[a]&1)
                    nega = !nega;
            
            #if 0
            cerr << ' ' << (nega ? '-' : '+') << ' ';
            for(unsigned a=0; a<depf; ++a)
                cerr << char('A' + a*depf + ps[a]);
            #endif
        
            thisresult = (tmptype) (nega ? -1 : 1);
           }
            for(unsigned a=0; a<depf; ++a)
                thisresult *= mat[a*depf + ps[a]];
            result += thisresult;
            
            while(++pc[level] >= depf-level)
            {
                if(!level)
                {
                    /* All done! */
                    #if 0
                    cerr << endl << endl;
                    #endif
                    return result;
                }
                --level;
            }
            #if 0
            if(level <= (depf<4 ? 0 : depf-4))cerr << endl;
            #endif
        Rotate:
            ps[level] = (ps[level] + 1) % depf;
        }
    }

public:
    void Guess(
        const vector<sourcetype> &keys,
        const vector<sourcetype> &values) const
    {
        if(verbose > 0)
        {
            cerr << "Analyzing {";
            for(unsigned a=0; a<keys.size(); ++a)
            {
                if(a) cerr << ',';
                cerr << keys[a];
            }
            cerr << "} => {";
            for(unsigned a=0; a<values.size(); ++a)
            {
                if(a) cerr << ',';
                cerr << values[a];
            }
            cerr << "}\n";
        }
        if(keys.size() != values.size())
        {
            cerr << "Error: keys.size() (" << keys.size() << ") != values.size() (" << values.size() << ")\n";
            return;
        }
        
        for(unsigned elemcount=1; elemcount<=keys.size(); ++elemcount)
        {
            /* Initialize the matrix using the keys. */
            vector<sourcetype> mat (elemcount * elemcount);
            
            vector<unsigned> keyindex(elemcount);
            for(unsigned a=0; a<elemcount; ++a)
            	keyindex[a] = a;
            
            tmptype biggesterror, smallesterror;
            vector<elem> bestguess, guess(elemcount);
            bool firsttest = true;
            for(;;)
            {
                for(unsigned a=0; a<elemcount; ++a)
                    for(unsigned b=0; b<elemcount; ++b)
                    {
                        sourcetype val = 1;
                        for(unsigned c=0; c<b; ++c)val *= keys[keyindex[a]];
                        mat[a*elemcount + b] = val; /* pow(key, b) */
                    }
                
                tmptype det = calcdet(mat, elemcount);
                
                typedef algofinder<sourcetype>::elem elem;
                
                /* Build the guess */
                for(unsigned a = 0; a < elemcount; ++a)
                {
                    /* Replace a column with the wanted values. */
                    vector<sourcetype> mat2 = mat;
                    
                    for(unsigned c = 0; c < elemcount; ++c)
                        mat2[c*elemcount+a] = values[keyindex[c]];
                    
                    guess[elemcount-1-a].integer  = calcdet(mat2, elemcount) / det;
                    guess[elemcount-1-a].exponent = a;
                }
                
                /* Test the guess */
                biggesterror = Test(keys, values, guess);
                
                if(verbose > 2)
                {
	                cerr << "One test (biggesterror = " << biggesterror << " (" << biggesterror.value() << "): ";
	                cerr << guess << endl;
	                cerr << "(Combination:";
	                for(unsigned a=0; a<elemcount; ++a)cerr << ' ' << keyindex[a];
	                cerr << ")\n";
                }
                
                if(maxerror <= 1e-10)
                {
                    bestguess = guess;
                    smallesterror = biggesterror;
                    break;
                }
                
                if(biggesterror < smallesterror || firsttest)
                {
                	bestguess = guess;
                    smallesterror = biggesterror;
                }
                
                firsttest = false;
                
                for(unsigned a=elemcount; a-- > 0; )
                {
                	if((a == elemcount-1 && keyindex[a] == keys.size()-1)
                	|| (a  < elemcount-1 && keyindex[a] == keyindex[a+1]-1))
                	{
                		if(!a) goto AllCombinationsDone;
                		continue;
                	}
                	++keyindex[a];
                	while(++a < elemcount)
                		keyindex[a] = keyindex[a-1]+1;
                	break;
                }
            }
        AllCombinationsDone:
            if(smallesterror.value() <= maxerror)
            {
                cout << bestguess << endl;
                if(verbose > 0)
                    cerr << "(Smallest biggest error = " << smallesterror << " (" << smallesterror.value() << ")\n";
                return;
            }
            if(verbose > 1)
            {
                cerr << "Bad guess (smallest biggest error = " << smallesterror << " (" << smallesterror.value() << "): ";
                cerr << bestguess << endl;
            }
        }
        if(verbose > 0)
            cerr << "Not found!\n";
    }

    template<typename itemtype>
    void Guess(
        const vector<itemtype> &keys,
        const vector<itemtype> &values) const
    {
        vector<sourcetype> fkeys(keys.size());
        vector<sourcetype> fvalues(keys.size());
        for(unsigned a=0; a<keys.size(); ++a)fkeys[a] = keys[a];
        for(unsigned a=0; a<values.size(); ++a)fvalues[a] = values[a];
        Guess(fkeys, fvalues);
    }

    vector<sourcetype> array(sourcetype a,sourcetype b,sourcetype c,sourcetype d) const
    {
        vector<sourcetype> result(4);
        result[0]=a;result[1]=b;result[2]=c;result[3]=d;
        return result;
    }

    void Test() const
    {
        /* Should output: x + 4 */     
        
        Guess( array(0,1,2,3),           /*    x*0 + y*1 = 4      x=(4*1-0*5) / (1-0)=4 */
               array(4,5,6,7) );         /*    x*1 + y*1 = 5      y=(5  -  4) / (1-0)=1 */
        
        /* Should output: 2x */

        Guess( array(10,11,12,13),       /*    x*10 + y*1 = 20    x=(20*11-22*10) / (11-10)=0 */
               array(20,22,24,26) );     /*    x*11 + y*1 = 22    y=(22   -20)    / (11-10)=2 */
        
        /* Should output: 3x + 1 */
        
        Guess( array(1,2,3,4),
               array(4,7,10,13) );
        
       /* Should output: xx - 1 */
        
        Guess( array(1,2,3,4),
               array(0,3,8,15) );

        /* Should output: xx - 2x +20 */
        
        Guess( array( 1, 2, 3, 4),    /*  x*1 + y*1 + z*1 = 19  x = 1*(2-3)+1(-9blah  */
               array(19,20,23,28) );  /*  x*4 + y*2 + z*1 = 20  it works anyway. :) */
                                      /*  x*9 + y*3 + z*1 = 23 */
        Guess( array( 1, 2, 3, 5),
               array(11,27,23,29) );
    }
};
#include <argh.hh>

int main(int argc, const char *const *argv)
{
    ParamHandler Argh;
    Argh.AddLong("help",    'h').SetBool().SetDesc("This help.");
    Argh.AddLong("version", 'V').SetBool().SetDesc("Displays version information.");
    Argh.AddLong("verbose", 'v').SetBool().SetDesc("More verbose.");
    Argh.AddLong("maxerror", 'e').SetFloat(0.0, 3e38).SetDesc("Set maximum error (absolute).");
    Argh.StartParse(argc, argv);
    bool anything = false;
    verbose = 0;
    for(;;)
    {
        int c = Argh.GetParam();
        if(c == -1)break;
        
        switch(c)
        {
            case 'V': printf("%s\n", VERSION); return 0;
            case 'v': if(Argh.GetBool())++verbose;else --verbose;
                      break;
            case 'e': maxerror = Argh.GetFloat(); break;
            case 'h': goto Usage;
            default:
            {
                typedef long double valtype;
                string s = Argh.GetString();
                unsigned a=0, b=s.size();
                vector<valtype> keys, values;
                bool tmpset=false, k=false;
                for(a=0; a<b; ++a)
                {
                    const char *begin = s.c_str() + a;
                    char *endptr;
                    
                    errno = 0;
                    valtype tmp = strtold(begin, &endptr);
                    //valtype tmp = strtod(begin, &endptr);
                    //valtype tmp = strtol(begin, &endptr, 10);
                    
                    if(errno == ERANGE)
                    {
                        /* FIXME: Handle this properly */
                        endptr = const_cast<char *> (begin);
                    }
                    
                    if(endptr == begin)
                    {
                        goto Got;
                    }
                    else
                    {
                        a += (endptr-begin)-1;
                        tmpset = true;
                    Got:
                        if(tmpset)
                            (k?values:keys).push_back(tmp);
                        tmpset = false;
                        if(a<b && s[a]=='=')k=!k;
                    }
                }
                
                algofinder<fraction<verylong> > gruu;
                gruu.Guess(keys, values);
                
                anything = true;
            }
        }
    }
    if(!Argh.ok())return -1;
    if(!anything)
    {
Usage:
        printf(
            "algoguesser v"VERSION" - Copyright (C) 1992,2001 Bisqwit (http://iki.fi/bisqwit/)\n"
            "Guesses formula\n"
            "\nUsage: %s [<option> [<...>]] [<serie>=<serie> [<...>]]\n"
            "\nSerie format: <number>[,<number>[<...>]]\n"
            "\nOptions:\n", argv[0]);
        Argh.ListOptions();
        return 0;
    }
    
//  algofinder<int> gruu;
//  gruu.Test();
    
    return 0;
}
